//
//  PokktManager.h
//  ObjectiveCTask
//
//  Created by Subhendu Sekhar Sahu on 01/07/16.
//  Copyright © 2016 Pokkt. All rights reserved.
//

#import <Foundation/Foundation.h>
#import <UIKit/UIKit.h>
#import "PokktConfig.h"
#import "PokktAdDelegate.h"
#import "PokktNetworkModel.h"
#import "PokktIAPDetails.h"
#import "PokktInitDelegate.h"
#import "PokktBannerDelegate.h"

/*!
 @abstract
 This is the main class of POKKT SDK. Developer application can interact with SDK via this
 class since SDK functionality are exposed by PokktManager.
 Developers should not try to access other SDK APIs directly.
 */

@interface PokktManager : NSObject


/*!
 @abstract
 This method initialises POKKT SDK and it should not be called again and again.
 
 This method sends a callback with POKKT SDK init result.
 
 @param pokktConfig {@link PokktConfig}
 
 Checks whether Application has all the necessary permission to proceed further.
 
 It then checks whether the application has all the SDK resource.
 
 Checks whether google play services are available.
 
 It uses PokktConfig object to initialise Store and UserDetails.
 
 Then it starts the session.
 
 It then fetches the network list from server.
 
 initPokkt will not be called again once initialized.
 
 This function sends a callback of PokktInitialised via delegate.

 */

+ (void)initPokkt: (PokktConfig *)pokktConfig;

/*!
 @abstract
 This method is used to update PokktConfig if needed. Normally it is not needed by developers.

 @param pokktConfig {@link PokktConfig}
 */

+ (void)updatePokktConfig:(PokktConfig *)pokktConfig;

/*!
 @abstract
 Set the Ad Delegate
 
 @param adDelegate Ad Delegate for the initPokkt instance
 
 @discussion Call this before calling initPokkt. SDK uses this delegate to communicate to your app and notifies the status of ads, such as an ad has been cached or failed, ad watched
     completely or skipped and ad availability status.

 */

+ (void)setAdDelegate:(id <PokktAdDelegate>)adDelegate;

/*!
 @abstract
 Set the PokktInit Delegate
 
 @param initDelegate PokktInit Delegate for the initPokkt instance
 
 @discussion Call this before calling initPokkt. SDK uses this delegate to communicate to your app and notifies the status of POKKT Initiatisation.
 */

+ (void)setInitDelegate:(id <PokktInitDelegate>)initDelegate;

/*!
 @abstract
 Set the Banner Delegate
 
 @param bannerDelegate Banner Delegate for the LoadBanner instance
 
 @discussion Call this before calling LoadBanner. SDK uses this delegate to communicate to your app
 and notifies the status of Banner.
 */

+ (void)setBannerDelegate:(id <PokktBannerDelegate>)bannerDelegate;

/*!
 @abstract
 Returns the version of the POKKT SDK.
 */

+ (NSString *)getSDKVersion;

/*!
 @abstract
 Returns the debug enable status of the POKKT SDK.
 */

+ (BOOL)isDebugEnabled;

/*!
 @abstract
 This method enables/disables the debug logging done by SDK.
 
 by default, logging is off.
 
 @param isDebug true if debug on else false
 */

+ (void)setDebug:(BOOL)isDebug;

/*!
 @abstract
 This method starts a new session.
 This is optional method that developer app can all.
 Even if developer does not calls this , initPokkt method call also calls startSession if session is not started.

 */

+ (void)startSession;

/*!
 @abstract
 This method sets an end time for  session to inform the server about the session end time.

 This is optional to call by developer application.
 
 */

+ (void)endSession;

/*!
 @abstract
 This method can be used by developer to check if there is any ad cached for a particular adConfig and is available to show.
 
 It only works if pokktManager is initialised successfully.
 
 @param adConfig {@link AdConfig}
 */

+ (void)checkAdAvailability:(PokktAdConfig *)adConfig;

/*!
 @abstract
 This method can be used by developer to get the virtual coins available for the  cached ad of a particular adConfig.
 
 It only works if PokktManager is initialised successfully.

 @param adConfig {@link AdConfig}
 
 @return ad coin value Default is 0
 */

+ (float)getRewardedAdVc:(PokktAdConfig *)adConfig;

/*!
 @abstract
 This method will be used to cache an ad of a particular adConfig.
 Ad can be video or interstitial depending on the adconfig value(adconfig.adFormat) such as '0' for Video, '3' for Interstitial.
 It only works if pokktManager is initialised successfully.

 @param adConfig {@link AdConfig}
 */

+ (void)cacheAd:(PokktAdConfig *)adConfig;

/*!
 @abstract
 This method will be used to show an ad of a particular adConfig.
 Developer should call checkAdAvailability to check if any ad is available to show.
 It only works if pokktManager is initialised successfully.

 @param viewController The UIViewController in which you will display the ad. Must not be `nil`. This is required so that the ad can present additional view controllers if the user interacts with it.
 @param adConfig {@link AdConfig}
 */

+ (void)showAd:(PokktAdConfig *)adConfig viewController:(UIViewController *)viewController;

/*!
 @abstract
 This method sends the developer app installation information to POKKT server.
 Developer has to call this method only once when his application installs on device.
 Pokktmanager init should be called before calling this method.

 */

+ (void)notifyAppInstall;

/*!
 @abstract
 This exposed method let developers export POKKT SDK log.

 @param viewController The UIViewController in which you will see all installed app on your device to export log file . Must not be `nil`.
 */

+ (void)exportLog:(UIViewController *)viewController;

/*!
 @abstract
 This function will send recieved notification data to POKKT server.
 
 @param localNotification received notification in AppDelegate.m class. You need to pass    same.
 
 */

+ (void)inAppNotificationEvent:(UILocalNotification *)localNotification;

/*!
 @abstract
 This function fetch all notification from Pokkt server. All fetched notification will fire on date which you set on POKKT Dashboard.
 
 */

+ (void)callBackgroundTaskCompletedHandler:(void (^)(UIBackgroundFetchResult result))completionHandler;

/*!
 @abstract
  This function returns the caching timeout of the particular network.
  After timeout, next network caching will be tried.
 
  @param networkModel {@link NetworkModel}
  @return Timeout
 */

+ (double)getCachingTimeoutDuration:(PokktNetworkModel *)networkModel;

/*!
 @abstract
 This is an utility method to show toast.
 
 @param viewController The UIViewController in which you will display the toast message. Must not be `nil`.
 @param message Message to show
 */

+ (void)showToast:(NSString *)message viewController:(UIViewController *)viewController;

/*!
 @abstract
 This method is called by developer to send any in-app purchase information to Pokkt.

 @param inAppPurchaseDetails {@link PokktIAPDetails}
 */

+ (void)trackIAP:(PokktIAPDetails *)inAppPurchaseDetails;

/*!
 @abstract
 This method is called by developer to register notification.
 
 @param aSelector Selector method to register notification.
 
 @param application {@link UIApplication}
 */

+ (void)setupNotification:(SEL)aSelector application:(UIApplication*)application;

/*!
 @abstract
 Load banner will load the banner and will display on screen.
 
 @param viewController The UIViewController in which you will display the banner. Must not be `nil`. This is required so that the ad can present additional view controllers if the user interacts with it.
 @param screenName      You will get this from Pokkt dashboard once you create banner placement so pass that screen name.
 @param bannerContainer is a view on that banner will display. It must be added on your viewController before loading banner.
 */

+(void)loadBanner:(NSString *)screenName bannerView:(UIView *)bannerContainer withViewController:(UIViewController *)viewController;

/*!
 @abstract
 Use setBannerAutoRefresh method to disable/enable auto refresh.
 
 Default it is true so banner will auto refresh in given interval in pokkt dashboard
 
 @param isAutoRefresh Set false if you want to disable auto refresh. Default is true
 */

+(void)setBannerAutoRefresh:(BOOL)isAutoRefresh;

/*!
 @abstract
 Use destroyBanner method use to remove your bannerContainer.
 
 @param bannerContainer your view which is used to show banner ad
 */

+(void)destroyBanner:(UIView *)bannerContainer;

/*!
 @abstract
 Use to add log in logfile.
 
 @param message text message
 */

+(void)printLog:(NSString *)message;

@end
